package com.fr.plugin.third.party.jsdibgd;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.report.chart.BaseChartWithData;
import com.fanruan.api.util.AssistKit;
import com.fanruan.api.util.IOKit;
import com.fanruan.api.util.StringKit;
import com.fr.base.Utils;
import com.fr.base.chart.cross.FormulaProcessor;
import com.fr.base.chart.result.WebChartIDInfo;
import com.fr.chart.ChartWebPara;
import com.fr.chart.ChartWebParaProvider;
import com.fr.extended.chart.HyperLinkPara;
import com.fr.general.FRFont;
import com.fr.general.GeneralUtils;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.third.party.jsdibgd.color.ColorUtils;
import com.fr.plugin.third.party.jsdibgd.data.DataColumnFieldCollection;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;
import com.fr.web.core.SessionPoolManager;
import com.fr.web.core.TemplateSessionIDInfo;

import java.awt.*;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;


public class CustomChartConfig extends BaseChartWithData {
    private static final String ID = "CUSTOM_MAP_CHART_JSDibgd";
    private static final String XML_TAG_NAME = "CUSTOM_MAP_CHART_JSDibgd_attr";

    private int selectedAreaColor = Color.YELLOW.getRGB();
    private int positionDynamicColor = Color.RED.getRGB();
    private FRFont stationNameFrFont = FRFont.getInstance();
    private int mapSymbolSize = 15;
    private int selectSymbolSize = 20;

    private boolean linkEnable = false;
    private String widgetNames = "";
    private String paraName = "stationName";

    public boolean isLinkEnable() {
        return linkEnable;
    }

    public void setLinkEnable(boolean linkEnable) {
        this.linkEnable = linkEnable;
    }

    public int getSelectSymbolSize() {
        return selectSymbolSize;
    }

    public void setSelectSymbolSize(int selectSymbolSize) {
        this.selectSymbolSize = selectSymbolSize;
    }

    public String getWidgetNames() {
        return widgetNames;
    }

    public void setWidgetNames(String widgetNames) {
        this.widgetNames = widgetNames;
    }

    public String getParaName() {
        return paraName;
    }

    public void setParaName(String paraName) {
        this.paraName = paraName;
    }

    public int getSelectedAreaColor() {
        return selectedAreaColor;
    }

    public void setSelectedAreaColor(int selectedAreaColor) {
        this.selectedAreaColor = selectedAreaColor;
    }

    public int getPositionDynamicColor() {
        return positionDynamicColor;
    }

    public void setPositionDynamicColor(int positionDynamicColor) {
        this.positionDynamicColor = positionDynamicColor;
    }

    public FRFont getStationNameFrFont() {
        if (stationNameFrFont == null) {
            return FRFont.getInstance();
        }
        return stationNameFrFont;
    }

    public void setStationNameFrFont(FRFont stationNameFrFont) {
        this.stationNameFrFont = stationNameFrFont;
    }

    public int getMapSymbolSize() {
        return mapSymbolSize;
    }

    public void setMapSymbolSize(int mapSymbolSize) {
        this.mapSymbolSize = mapSymbolSize;
    }

    @Override
    protected Image designImage(int width, int height, int resolution, ChartWebParaProvider chartWebPara) {
        return IOKit.readImageWithCache("com/fr/plugin/third/party/jsdibgd/images/chart_type_demo.png");
    }

    @Override
    protected Image exportImage(int width, int height, int resolution, ChartWebParaProvider chartWebPara) {
        return null;
    }

    @Override
    public CustomChartConfig clone() throws CloneNotSupportedException {
        CustomChartConfig result = (CustomChartConfig) super.clone();
        result.setSelectedAreaColor(this.getSelectedAreaColor());
        result.setPositionDynamicColor(this.getPositionDynamicColor());
        result.setStationNameFrFont(this.getStationNameFrFont());
        result.setMapSymbolSize(this.getMapSymbolSize());

        result.setSelectSymbolSize(this.getSelectSymbolSize());
        result.setWidgetNames(this.getWidgetNames());
        result.setParaName(this.getParaName());

        result.setLinkEnable(this.isLinkEnable());
        return result;
    }

    @Override
    public int hashCode() {
        return super.hashCode() + AssistKit.hashCode(this.getSelectedAreaColor()) + AssistKit.hashCode(this.getPositionDynamicColor()) + AssistKit.hashCode(this.getStationNameFrFont())
                + AssistKit.hashCode(this.getMapSymbolSize())
                + AssistKit.hashCode(this.getSelectSymbolSize())
                + AssistKit.hashCode(this.getWidgetNames())
                + AssistKit.hashCode(this.getParaName())

                + AssistKit.hashCode(this.isLinkEnable())
                ;
    }

    @Override
    public boolean equals(Object ob) {
        return super.equals(ob)
                && ob instanceof CustomChartConfig
                && AssistKit.equals(this.getSelectedAreaColor(), ((CustomChartConfig) ob).getSelectedAreaColor())
                && AssistKit.equals(this.getPositionDynamicColor(), ((CustomChartConfig) ob).getPositionDynamicColor())
                && AssistKit.equals(this.getStationNameFrFont(), ((CustomChartConfig) ob).getStationNameFrFont())
                && AssistKit.equals(this.getMapSymbolSize(), ((CustomChartConfig) ob).getMapSymbolSize())

                && AssistKit.equals(this.getSelectSymbolSize(), ((CustomChartConfig) ob).getSelectSymbolSize())
                && AssistKit.equals(this.getWidgetNames(), ((CustomChartConfig) ob).getWidgetNames())
                && AssistKit.equals(this.getParaName(), ((CustomChartConfig) ob).getParaName())

                && AssistKit.equals(this.isLinkEnable(), ((CustomChartConfig) ob).isLinkEnable())
                ;
    }

    @Override
    public JSONObject createAttributeConfig(ChartWebParaProvider chartWebPara) {
        JSONObject jsonObject = super.createAttributeConfig(chartWebPara);
        jsonObject.put("dataStatus", false);
        if (isDatasEmpty()) {
            return jsonObject;
        }
        //添加认证
        if (!PluginContexts.currentContext().isAvailable()) {
            LogKit.error("The custom map plugin trial is over, please purchase a license");
            return jsonObject;
        }

        JSONArray datasJson = createDatas();
        jsonObject.put("configDatas", datasJson);

        JSONObject mapConfigJson = new JSONObject();
        mapConfigJson.put("selectedAreaColor", ColorUtils.rgbToHex(new Color(getSelectedAreaColor())));
        mapConfigJson.put("positionDynamicColor", ColorUtils.rgbToHex(new Color(getPositionDynamicColor())));
        mapConfigJson.put("mapSymbolSize", getMapSymbolSize());
        mapConfigJson.put("selectSymbolSize", getSelectSymbolSize());
        jsonObject.put("mapConfig", mapConfigJson);

        JSONObject fontConfigJson = new JSONObject();
        FRFont tempFrFont = getStationNameFrFont();
        fontConfigJson.put("name", tempFrFont.getFamily());
        fontConfigJson.put("size", Utils.round5(tempFrFont.getSize2D()));
        fontConfigJson.put("color", ColorUtils.rgbToHex(tempFrFont.getForeground()));
        jsonObject.put("stationNameFontConfig", fontConfigJson);

        jsonObject.put("uuid", UUID.randomUUID().toString());

        JSONObject linkageConfigJson = createLinkJson(chartWebPara);
        jsonObject.put("linkageConfig", linkageConfigJson);

        jsonObject.put("dataStatus", true);
        return jsonObject;
    }


    @Override
    public void dealFormula(FormulaProcessor formulaProcessor) {
        super.dealFormula(formulaProcessor);
    }

    @Override
    public String getID() {
        return ID;
    }

    @Override
    public void readAttr(XMLableReader xmLableReader) {
        super.readAttr(xmLableReader);
        setSelectedAreaColor(xmLableReader.getAttrAsInt("selectedAreaColor", Color.YELLOW.getRGB()));
        setPositionDynamicColor(xmLableReader.getAttrAsInt("positionDynamicColor", Color.RED.getRGB()));
        setMapSymbolSize(xmLableReader.getAttrAsInt("mapSymbolSize", 10));

        setSelectSymbolSize(xmLableReader.getAttrAsInt("selectSymbolSize", 15));
        setWidgetNames(xmLableReader.getAttrAsString("widgetNames", ""));
        setParaName(xmLableReader.getAttrAsString("paraName", "stationName"));

        setLinkEnable(xmLableReader.getAttrAsBoolean("linkEnable", false));

        FRFont tempFrFont = FRFont.getInstance();
        String fontName = xmLableReader.getAttrAsString("stationNameFontName", StringKit.EMPTY);
        if (StringKit.isNotEmpty(fontName)) {
            tempFrFont = tempFrFont.applyName(fontName);
        }
        String fontSize = xmLableReader.getAttrAsString("stationNameFontSize", "10");
        tempFrFont = tempFrFont.applySize(Float.parseFloat(fontSize));
        tempFrFont = tempFrFont.applyForeground(new Color(xmLableReader.getAttrAsInt("stationNameFontColor", 0)));
        setStationNameFrFont(tempFrFont);
    }

    @Override
    public void writeAttr(XMLPrintWriter xmlPrintWriter) {
        super.writeAttr(xmlPrintWriter);
        xmlPrintWriter.attr("selectedAreaColor", getSelectedAreaColor());
        xmlPrintWriter.attr("positionDynamicColor", getPositionDynamicColor());
        xmlPrintWriter.attr("mapSymbolSize", getMapSymbolSize());

        xmlPrintWriter.attr("selectSymbolSize", getSelectSymbolSize());
        xmlPrintWriter.attr("widgetNames", getWidgetNames());
        xmlPrintWriter.attr("paraName", getParaName());

        xmlPrintWriter.attr("linkEnable", isLinkEnable());

        FRFont tempFrFont = getStationNameFrFont();
        xmlPrintWriter.attr("stationNameFontName", tempFrFont.getFamily());
        xmlPrintWriter.attr("stationNameFontSize", Utils.round5(tempFrFont.getSize2D()));
        xmlPrintWriter.attr("stationNameFontColor", tempFrFont.getForeground().getRGB());
    }

    private JSONObject createLinkJson(ChartWebParaProvider chartWebParaProvider) {
        JSONObject disableLinkJson = new JSONObject();
        disableLinkJson.put("linkEnable", false);
        if (!isLinkEnable()) {
            return disableLinkJson;
        }

        if (chartWebParaProvider == null) {
            return disableLinkJson;
        }

        if (!(chartWebParaProvider instanceof ChartWebPara)) {
            return disableLinkJson;
        }

        String pName = trim(getParaName());
        if (StringKit.isEmpty(pName)) {
            return disableLinkJson;
        }

        ChartWebPara chartWebPara = (ChartWebPara) chartWebParaProvider;
        String ownerChartName = getOwnerChartName(chartWebPara);
        List<String> validWidgetNames = getValidWidgetNames(ownerChartName);
        if (validWidgetNames.size() <= 0) {
            return disableLinkJson;
        }

        String bookPath = getBookPath(chartWebPara);

        JSONArray widgetArray = new JSONArray();
        for (int i = 0, max = validWidgetNames.size() - 1; i <= max; i++) {
            widgetArray.add(validWidgetNames.get(i));
        }
        JSONObject linkJson = new JSONObject();
        linkJson.put("widgetNames", widgetArray);
        linkJson.put("paraName", pName);
        linkJson.put("bookPath", bookPath);
        linkJson.put("linkEnable", true);
        return linkJson;
    }

    private String getOwnerChartName(ChartWebPara chartWebPara) {
        if (chartWebPara == null) {
            return "";
        }
        WebChartIDInfo chartIDInfo = chartWebPara.getChartIDInfo();
        String ownerChartName = chartIDInfo.getName();
        if (StringKit.isNotEmpty(ownerChartName)) {
            ownerChartName = ownerChartName.toUpperCase();
        }
        return ownerChartName;
    }

    private String getBookPath(ChartWebPara chartWebPara) {
        if (chartWebPara == null) {
            return "";
        }
        String sessionID = chartWebPara.getSessionID();
        TemplateSessionIDInfo templateSessionIDInfo = (TemplateSessionIDInfo) SessionPoolManager.getSessionIDInfor(sessionID, TemplateSessionIDInfo.class);
        if (templateSessionIDInfo == null) {
            return "";
        }
        String path = templateSessionIDInfo.getRelativePath();
        return path;
    }

    private List<String> getValidWidgetNames(String ownerChartName) {
        List<String> names = new ArrayList<>();

        String wName = trim(getWidgetNames());
        if (StringKit.isEmpty(wName)) {
            return names;
        }

        wName = wName.replace("，", ",");
        String[] tempNames = wName.split(",");
        if ((tempNames == null) || (tempNames.length <= 0)) {
            return names;
        }

        String tempName;
        for (int i = 0, max = tempNames.length - 1; i <= max; i++) {
            tempName = tempNames[i];
            tempName = trim(tempName);
            if (StringKit.isEmpty(tempName)) {
                continue;
            }
            tempName = tempName.toUpperCase();
            if (tempName.equals(ownerChartName)) {
                continue;
            }

            if (names.contains(tempName)) {
                continue;
            }
            names.add(tempName);
        }
        return names;
    }

    private String trim(String value) {
        String tempValue = StringKit.trim(value);
        if (StringKit.isEmpty(tempValue)) {
            return "";
        }
        return tempValue;
    }

    private List<Integer> toColorList(String content) {
        List<Integer> tempColors = new ArrayList<>();
        if (StringKit.isEmpty(content)) {
            return tempColors;
        }
        content = content.trim();
        if (StringKit.isEmpty(content)) {
            return tempColors;
        }
        String[] tempArray = content.split(",");
        if ((tempArray == null) || (tempArray.length <= 0)) {
            return tempColors;
        }
        String tempValue;
        int tempColorValue;
        for (int i = 0, max = tempArray.length - 1; i <= max; i++) {
            tempValue = tempArray[i];
            tempColorValue = Integer.valueOf(tempValue);
            tempColors.add(tempColorValue);
        }

        return tempColors;
    }


    private int getObjectValue(List<Object> objs) {
        double tempValue = getObjectDoubleValue(objs);
        Double doubleValue = new Double(tempValue);
        int value = doubleValue.intValue();
        return value;
    }

    private double getObjectDoubleValue(List<Object> objs) {
        double value = 0;
        if ((objs == null) || (objs.size() <= 0)) {
            return -1;
        }
        try {
            value = Double.valueOf(String.valueOf(objs.get(0)));
        } catch (Exception e) {
            value = -1;
        }

        return value;
    }


    private double getFinishRate(double monthFinishValue, double maxValue) {
        double f = monthFinishValue * 1.00 / maxValue;
        BigDecimal b = new BigDecimal(f);
        double f1 = b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        return f1;
    }


    private JSONArray createDatas() {
        JSONArray datasJson = new JSONArray();
        DataColumnFieldCollection columnFieldCollection = getFieldCollection(DataColumnFieldCollection.class);
        List<Object> areaNameValues = columnFieldCollection.getAreaName().getValues();
        List<Object> deptNameValues = columnFieldCollection.getDeptName().getValues();
        List<Object> outputValueValues = columnFieldCollection.getOutputValue().getValues();
        List<Object> profitValues = columnFieldCollection.getProfit().getValues();
        List<Object> cashFlowValues = columnFieldCollection.getCashFlow().getValues();
        List<Object> newContractValues = columnFieldCollection.getNewContract().getValues();
        List<Object> longitudeValues = columnFieldCollection.getLongitude().getValues();
        List<Object> latitudeValues = columnFieldCollection.getLatitude().getValues();

        JSONObject itemJson;
        JSONArray placeJson;
        String areaName,areaName1;
        for (int i = 0, max = areaNameValues.size() - 1; i <= max; i++) {
            itemJson = new JSONObject();
            placeJson = new JSONArray();
            areaName = getColumnValue(i, areaNameValues);
            areaName1 = DataFactory.getAbbreviatedProvince(areaName);
            itemJson.put("dataIndex", i);
            itemJson.put("areaName", areaName);
            itemJson.put("areaName1", areaName1);
            itemJson.put("deptName", getColumnValue(i, deptNameValues));
            itemJson.put("outputValue", getColumnValue(i, outputValueValues));
            itemJson.put("profit", getColumnValue(i, profitValues));
            itemJson.put("cashFlow", getColumnValue(i, cashFlowValues));
            itemJson.put("newContract", getColumnValue(i, newContractValues));
            placeJson.add(toDouble(getColumnValue(i, longitudeValues)));
            placeJson.add(toDouble(getColumnValue(i, latitudeValues)));
            itemJson.put("place",placeJson);
            datasJson.add(itemJson);
        }
        return datasJson;
    }

    private double toDouble(Object obj) {
        if (obj == null) {
            return 0;
        }
        String tempStr = String.valueOf(obj);
        if (StringKit.isEmpty(tempStr)) {
            return 0;
        }
        double value = Double.valueOf(tempStr);
        return value;
    }

    private String getColumnValue(int index, List<Object> values) {
        if (isListEmpty(values) || (index >= values.size())) {
            return "";
        }

        String value = GeneralUtils.objectToString(values.get(index));
        value = StringKit.trim(value);
        if (StringKit.isEmpty(value)) {
            return "";
        }
        return value;
    }


    private boolean isListEmpty(List list) {
        if ((list == null) || (list.size() <= 0)) {
            return true;
        }
        return false;
    }


    private boolean isDatasEmpty() {
        DataColumnFieldCollection columnFieldCollection = getFieldCollection(DataColumnFieldCollection.class);
        List<Object> areaNameValues = columnFieldCollection.getAreaName().getValues();
        if ((areaNameValues == null) || (areaNameValues.size() <= 0)) {
            return true;
        }
        return false;
    }


    private static final HyperLinkPara deptNameHyperLinkPara = new HyperLinkPara() {
        @Override
        public String getName() {
            return "部门名称";
        }

        @Override
        public String getFormulaContent() {
            return "deptName";
        }

        @Override
        public String[] getProps() {
            return new String[]{"data", "deptName"};
        }
    };

    @Override
    protected HyperLinkPara[] hyperLinkParas() {
        return new HyperLinkPara[]{
                deptNameHyperLinkPara,
        };
    }
}
