package com.fr.plugin.third.party.jsdibgd.color;


import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class ColorUtils {
    /**
     * 颜色变暗
     *
     * @param color
     * @return
     */
    public static Color toDark(Color color) {
        if (color == null) {
            return Color.black;
        }
        float[] hsb = rgb2hsb(color.getRed(), color.getGreen(), color.getBlue());
        float h = hsb[0];
        float s = hsb[1];
        float b = hsb[2];
        if (s >= 0.5) {
            s = (float) (s - 0.5);
        } else {
            s = 0;
        }
        int[] rgb = hsb2rgb(h, s, b);
        Color tempColor = new Color(rgb[0], rgb[2], rgb[2]);
        return tempColor;
    }

    public static float[] rgb2hsb(int rgbR, int rgbG, int rgbB) {
        assert 0 <= rgbR && rgbR <= 255;
        assert 0 <= rgbG && rgbG <= 255;
        assert 0 <= rgbB && rgbB <= 255;
        int[] rgb = new int[]{rgbR, rgbG, rgbB};
        Arrays.sort(rgb);
        int max = rgb[2];
        int min = rgb[0];

        float hsbB = max / 255.0f;
        float hsbS = max == 0 ? 0 : (max - min) / (float) max;

        float hsbH = 0;
        if (max == rgbR && rgbG >= rgbB) {
            hsbH = (rgbG - rgbB) * 60f / (max - min) + 0;
        } else if (max == rgbR && rgbG < rgbB) {
            hsbH = (rgbG - rgbB) * 60f / (max - min) + 360;
        } else if (max == rgbG) {
            hsbH = (rgbB - rgbR) * 60f / (max - min) + 120;
        } else if (max == rgbB) {
            hsbH = (rgbR - rgbG) * 60f / (max - min) + 240;
        }

        return new float[]{hsbH, hsbS, hsbB};
    }

    public static int[] hsb2rgb(float h, float s, float v) {
        assert Float.compare(h, 0.0f) >= 0 && Float.compare(h, 360.0f) <= 0;
        assert Float.compare(s, 0.0f) >= 0 && Float.compare(s, 1.0f) <= 0;
        assert Float.compare(v, 0.0f) >= 0 && Float.compare(v, 1.0f) <= 0;

        float r = 0, g = 0, b = 0;
        int i = (int) ((h / 60) % 6);
        float f = (h / 60) - i;
        float p = v * (1 - s);
        float q = v * (1 - f * s);
        float t = v * (1 - (1 - f) * s);
        switch (i) {
            case 0:
                r = v;
                g = t;
                b = p;
                break;
            case 1:
                r = q;
                g = v;
                b = p;
                break;
            case 2:
                r = p;
                g = v;
                b = t;
                break;
            case 3:
                r = p;
                g = q;
                b = v;
                break;
            case 4:
                r = t;
                g = p;
                b = v;
                break;
            case 5:
                r = v;
                g = p;
                b = q;
                break;
            default:
                break;
        }
        return new int[]{(int) (r * 255.0), (int) (g * 255.0),
                (int) (b * 255.0)};
    }

    /**
     * 获取渐变颜色 当前颜色到 红绿蓝 渐变
     *
     * @param color
     * @param count
     * @return
     */
    public static List<Color> getGradientColors(Color color, int count) {
        List<Color> colors = new ArrayList<>();
        if (count <= 0) {
            return colors;
        }
        if (count <= 1) {
            count = 1;
        }
        int max = count - 1;
        if (color == null) {
            for (int i = 0; i <= max; i++) {
                colors.add(Color.black);
            }
            return colors;
        }
        if (count == 1) {
            colors.add(color);
            return colors;
        }

        Color endColor = new Color(220, 220, 220);
        endColor = Color.RED;

        int maxStep = Math.abs(Math.abs(color.getRGB()) - Math.abs(endColor.getRGB()));
        int tempStep = Math.abs(Math.abs(color.getRGB()) - Math.abs(Color.GREEN.getRGB()));
        if (tempStep >= maxStep) {
            maxStep = tempStep;
            endColor = Color.GREEN;
        }

        tempStep = Math.abs(Math.abs(color.getRGB()) - Math.abs(Color.BLUE.getRGB()));
        if (tempStep >= maxStep) {
            maxStep = tempStep;
            endColor = Color.BLUE;
        }
        endColor = new Color(220, 220, 220);
        double redStep = (endColor.getRed() - color.getRed()) / (count + 1);
        double greenStep = (endColor.getGreen() - color.getGreen()) / (count + 1);
        double blueStep = (endColor.getBlue() - color.getBlue()) / (count + 1);

        int redValue = color.getRed();
        int greenValue = color.getGreen();
        int blueValue = color.getBlue();

        double redTempValue = color.getRed();
        double greenTempValue = color.getGreen();
        double blueTempValue = color.getBlue();

        Color tempColor = null;
        for (int i = 0; i <= max; i++) {
            if (redValue >= 255) {
                redValue = 255;
            }
            if (greenValue >= 255) {
                greenValue = 255;
            }
            if (blueValue >= 255) {
                blueValue = 255;
            }
            tempColor = new Color(redValue, greenValue, blueValue);
            colors.add(tempColor);
            redTempValue = (redTempValue + redStep);
            greenTempValue = (greenTempValue + greenStep);
            blueTempValue = (blueTempValue + blueStep);
            redValue = (int) redTempValue;
            greenValue = (int) greenTempValue;
            blueValue = (int) blueTempValue;
        }
        return colors;
    }

    public static String rgbToHex(Color color) {
        if (color == null) {
            return "#000000";
        }
        String value = "#" + String.format("%02X", color.getRed()) + String.format("%02X", color.getGreen()) + String.format("%02X", color.getBlue());
        return value;
    }



}
